/* eslint-disable */
import { Http, IDictionary, Metadata } from '@zeedhi/core';
import {
  AlertService, Card, Form, Select, Switch, Text,
} from '@zeedhi/common';
import ZdListPicker from '@/components/listPicker/ZdListPicker';
import { IterableComponentRender } from '@zeedhi/teknisa-components-common';

export class NewRelatorioController {
  // Funçao que seta as informações no select de 'Fonte de dados'
  public setFontdaos({ component }:{ component: Select }){
    Http.get('/getDataBase').then((response: IDictionary) => {
      response.data.dataset.dataBase.forEach((element: IDictionary) => {
        component.datasource.data.push({
          name: element.NMFONTEDADOSGRAFREL,
          value: element.NRFONTEDADOSGRAFREL,
        });
      });
    });
  }

  // Funcção que envia requisição após selecionar uma fonte de dados para preencher o campo com as colunas
  public setColDados({ component }:{ component: Select }){
    Http.get(`/getData?NR=${component.value}`).then((response: IDictionary) => {
      const dualList = Metadata.getInstance<ZdListPicker>('ComposGrafico');
      dualList.data = [];
      dualList.value = [];

      // Torna o preview invisivel
      this.previewFieldVisibility(false);

      response.data.dataset.data.forEach((element: IDictionary) => {
        dualList.data.push({
          nome: element.NMDADOSGRAFREL,
          id: element.NRDADOSGRAFREL,
        });
      });
    });
  }

  // Aciona ao mexer no switch de "Relatório publico"
  public switchPublic({ component }:{ component:Switch }) {
    // instancia o campo de funcionarios com permição para ver o relatorio
    const funcionariosAutorizados = Metadata.getInstance<Select>('SelectMultipleUsuarioComAcessoCardDadosDoRelatorio')
    // Ativa/Desativa o campo onde e selecionado os usuário que vão ter permissão para ver o relatório
    funcionariosAutorizados.disabled = !component.value;
    // verifica se esta desativando ou ativando para que torna ele obrigatorio ou não
    if (!component.value) {
      funcionariosAutorizados.removeValidation('required');
    } else {
      funcionariosAutorizados.addValidation('required')
    }

    // Define o texto ao lado do Switch
    component.label = (component.value)? 'Não' : 'Sim';
  }

  // Funcção responsavel por gerar as informações dentro do card para que seja renderizado um preview
  public preView({ component }:{ component: ZdListPicker }) {
    const condition = component.value.length  > 0;
    this.previewFieldVisibility(condition);
    if (condition) {
      const medida = Math.trunc(100/component.value.length);
      const iterable = Metadata.getInstance<IterableComponentRender>('IterablecomponentpreView');
      const dadosComNomeDaCol = component.data;
      iterable.datasource.data = [];
      component.value.forEach((element: number) => {
        iterable.datasource.data.push({
          medida: `${medida}%`,
          id: element,
          nome: dadosComNomeDaCol[dadosComNomeDaCol.findIndex((dados: IDictionary) => dados.id === element)].nome,
        });
      });
    }
  }

  // Função responsável por fazer campos de previsualização das colunas ficar visivel ou invisivel
  public previewFieldVisibility(visibility: boolean) {
    // Torna o texto visivel/invisivel
    Metadata.getInstance<Text>('SubTituloCardColunasParaORelatorio').isVisible = visibility

    // Torna o Card visivel/invisivel
    Metadata.getInstance<Card>('CardPreVsualização').isVisible = visibility;;
  }

  public buttonGerarRelatorio() {
    // Instancia form do primeiro card
    const firstForm = Metadata.getInstance<Form>("FormCardDadosDoRelatorio").validate();
    // Instancia form do terceiro card
    const secondForm = Metadata.getInstance<Form>("FormCardPeriodoDeRelatorio").validate();

    // Verifica se os dois forms são validos
    // ( tem que adicionar o criterio para validação das data secelecionadas e se a colunas selecionadas no arquivo)
    if (firstForm && secondForm) {
      // Aqui vai ficar os dados a ser enviado para salvar no banco
      console.log("validos");
    }
  }

  //Essa função sera adicionada ao evento de input ou change do ZdDate para avaliar se a data esta dentro do padrão
  // public msgErroData() {
  //   if () {
  //     AlertService.show({
  //       name: 'basic-alert',
  //       text: 'Data selecionada não pode ser superior a data atual!',
  //       top: true,
  //       left: false,
  //       timeout: 3000,
  //       color: "warning",
  //     });
  //   }
  // }
}
