/* eslint-disable no-console */
/* eslint-disable */
import {
  Metadata, IEventParam, IDictionary, Http, Loader, Router,
} from '@zeedhi/core';
import {
  Form, Chip, Text, Select, SelectMultiple, TextInput, AlertService,
} from '@zeedhi/common';
import { IterableComponentRender } from '@zeedhi/teknisa-components-common';

export class NovoChecklistController {
  public controllerChecklist = Loader.getInstance('ChecklistController');

  public dadosTarefa: IDictionary = {
    selectLocal: [],
    selectItens: [],
  };

  public dadoResposta: IDictionary = [];

  public dadosChip: IDictionary = {};

  public setDados(nome: string, cod: number) {
    return {
      name: nome,
      value: cod,
    };
  }

  public setLocal() {
    Http.get('/getLocalChk').then((response: IDictionary) => {
      response.data.dataset.local.forEach((dados: IDictionary) => {
        this.dadosTarefa.selectLocal.push(this.setDados(dados.NMTIPOLOCALCLI, dados.NRTIPOLOCALCLI));
      });
    });
  }

  public selectInformacaoItens() {
    this.setItens().then(() => {
      this.controllerChecklist.getEditDados().then((resposta: IDictionary) => {
        this.dadoResposta = resposta;
      });
    });
  }

  public setItens() {
    return new Promise((resolve, reject) => {
      try {
        Http.get('/filtroItens').then((response: IDictionary) => {
          response.data.dataset.filtro.forEach((dados: IDictionary) => {
            this.dadosTarefa.selectItens.push(this.setDados(dados.NMATIVIDADE, dados.NRATIVIDADE));
          });
          resolve();
        });
      } catch (error) {
        reject(error);
      }
    });
  }

  public async setTag() {
    Metadata.getInstance<IterableComponentRender>('iterable_parent').datasource.data = [];
    const tarefa = Metadata.getInstance<SelectMultiple>('selectTagTarefas');
    tarefa.datasource.data = [];
    tarefa.setValue([]);
    const dadosTarefas = await Http.post('/filtroTags', Metadata.getInstance<SelectMultiple>('selectItens').value);
    dadosTarefas.data.dataset.filtroTags.forEach((dados: IDictionary) => {
      tarefa.datasource.data.push({
        name: dados.NMTAREFA,
        value: dados.NRTAREFA,
        parent: dados.NRATIVIDADE,
        parentName: dados.NMATIVIDADE,
      });
    });
    console.log(this.dadoResposta.tarefa.length);
    if (this.dadoResposta.tarefa.length > 0) {
      tarefa.setValue(this.dadoResposta.tarefa);
      this.dadoResposta.tarefa = [];
    }
  }

  get descricaoNovoChecklist() {
    const descricao = Metadata.getInstance<Text>('descriçaoChecklist');
    return descricao;
  }

  get itensNovoChecklist() {
    const itens = Metadata.getInstance<Text>('itensNovoChecklist');
    return itens;
  }

  public getDadosInfo({ component }: IEventParam<Select>) {
    component.datasource.data = this.dadosTarefa[component.name];
  }

  get tarefaCheckList() {
    const tarefa = Metadata.getInstance<Select>('SelectTarefas');
    return tarefa.value;
  }

  public getParent(component: IEventParam<any>, nameComponent: string): any {
    if (component.component === nameComponent) {
      return component;
    }
    if (component.parent) {
      return this.getParent(component.parent, nameComponent);
    }
    return null;
  }

  public botaoCriarTarefa() {
    const form = Metadata.getInstance<Form>('formCadastroNovoChecklist');
    if (form.validate()) {
      form.resetValidation();
    }
  }

  public selecionaGrupoCriterio({ component }:{ component:Chip }) {
    component.outlined = !component.outlined;
    const conjunto = component.name.split('_');
    if (!component.outlined) {
      if (Object.keys(this.dadosChip).includes(conjunto[1])) {
        if (Object.keys(this.dadosChip[conjunto[1]]).includes(conjunto[2])) {
          this.dadosChip[conjunto[1]][conjunto[2]].push(conjunto[3]);
        } else {
          this.dadosChip[conjunto[1]][conjunto[2]] = [];
          this.dadosChip[conjunto[1]][conjunto[2]].push(conjunto[3]);
        }
      } else {
        this.dadosChip[conjunto[1]] = {};
        this.dadosChip[conjunto[1]][conjunto[2]] = [];
        this.dadosChip[conjunto[1]][conjunto[2]].push(conjunto[3]);
      }
    } else {
      this.dadosChip[conjunto[1]][conjunto[2]].splice(this.dadosChip[conjunto[1]][conjunto[2]].indexOf(Number(conjunto[3])), 1);
      if (this.dadosChip[conjunto[1]][conjunto[2]].length === 1) {
        delete this.dadosChip[conjunto[1]][conjunto[2]];
      }
      if (!Object.keys(this.dadosChip[conjunto[1]]).length) {
        delete this.dadosChip[conjunto[1]];
      }
    }
  }

  public inicializarDados() {
    this.setLocal();
    this.selectInformacaoItens();
  }

  public modeloCardCriterio(titulo: string, id: string) {
    return {
      name: '',
      component: 'ZdRow',
      cssClass: 'zd-pb-5',
      children: [
        {
          name: '',
          component: 'ZdCol',
          children: [
            {
              name: '',
              component: 'ZdRow',
              children: [
                {
                  name: '',
                  component: 'ZdCard',
                  children: [
                    {
                      name: '',
                      component: 'ZdText',
                      text: `${titulo}`,
                    },
                    {
                      name: id,
                      component: 'ZdRow',
                      events: {
                        onCreated: '{{NovoChecklistController.teste}}',
                      },
                      children: [],
                    },
                  ],
                },
              ],
            },
          ],
        },
      ],
    };
  }

  public modeloTag(tarefas: string) {
    return ({
      name: 'chipResult',
      component: 'ZdChip',
      label: `${tarefas}`,
      cssClass: 'zd-my-2',
      color: 'info',
    });
  }

  public dadosData: IDictionary = [];

  public renderizaTarefas({ component }: IEventParam<SelectMultiple>) {
    Metadata.getInstance<IterableComponentRender>('iterable_parent').datasource.data = [];
    this.dadosData = [];
    if (component.value.length) {
      Http.post('/getGrupo', component.value).then((response: IDictionary) => {
        response.data.dataset.grupos.forEach((dados: IDictionary) => {
          this.dadosData.push({
            DSTAGSTAREFA: dados.DSTAGSTAREFA,
            list: dados.list,
          });
        });
      }).then(() => {
        if (this.dadoResposta.chip && this.dadoResposta.chip.length) {
          this.dadosData.forEach((dados: IDictionary) => {
            dados.list.forEach((lista: IDictionary) => {
              const validacao = this.dadoResposta.chip.filter((dadosChipEdit: IDictionary) => dadosChipEdit.PARENT === lista.NRTAREFA && dadosChipEdit.tipo === dados.DSTAGSTAREFA);
              if (validacao.length) {
                lista.list.forEach((ativo: IDictionary) => {
                  if ((validacao.filter((dadosChipEdit: IDictionary) => ativo.NRCRITERIOTAREFA === dadosChipEdit.NRCRITERIOTAREFA)).length) {
                    ativo.ATIVO = true;
                    if (Object.keys(this.controllerChecklist.dadosChip).includes(dados.DSTAGSTAREFA)) {
                      if (Object.keys(this.controllerChecklist.dadosChip[dados.DSTAGSTAREFA]).includes(String(lista.NRATIVIDADE))) {
                        this.controllerChecklist.dadosChip[dados.DSTAGSTAREFA][lista.NRATIVIDADE].push(ativo.NRCRITERIOTAREFA);
                      } else {
                        this.controllerChecklist.dadosChip[dados.DSTAGSTAREFA][lista.NRATIVIDADE] = [];
                        this.controllerChecklist.dadosChip[dados.DSTAGSTAREFA][lista.NRATIVIDADE].push(ativo.NRCRITERIOTAREFA);
                      }
                    } else {
                      this.controllerChecklist.dadosChip[dados.DSTAGSTAREFA] = {};
                      this.controllerChecklist.dadosChip[dados.DSTAGSTAREFA][lista.NRATIVIDADE] = [];
                      this.controllerChecklist.dadosChip[dados.DSTAGSTAREFA][lista.NRATIVIDADE].push(ativo.NRCRITERIOTAREFA);
                    }
                  }
                });
              }
            });
          });
        }
      }).then(() => {
        Metadata.getInstance<IterableComponentRender>('iterable_parent').datasource.data = this.dadosData as IDictionary[];
      });
    }
  }

  public consoleLog(variavel: string) {
    const titulo = Metadata.getInstance<Text>('tituloNovaCheckList');
    titulo.text = variavel;
  }

  public insertNovoChecklistBase() {
    const selectObjectForm = Metadata.getInstance<Form>('FormPrincipal');
    if (selectObjectForm.validate()) {
      if (Object.keys(this.dadosChip).length) {
        const arrayDados = {
          tipoLocal: Metadata.getInstance<Select>('selectLocal').value,
          nomeCheckList: Metadata.getInstance<TextInput>('nomeCheckList').value,
          descricaoChecklist: Metadata.getInstance<TextInput>('descricaoChecklist').value,
          itensChecklist: Metadata.getInstance<SelectMultiple>('selectItens').value,
          selectTagTarefas: Metadata.getInstance<SelectMultiple>('selectTagTarefas').value,
          dadosChip: this.dadosChip,
        };
        Http.post('/insertChecklist', { DADOS: arrayDados, NRORG: 20 }).then(() => {
          Router.replace('/checklist');
        });
      } else {
        AlertService.show({
          name: 'basic-alert',
          text: 'Favor preencher pelo menos um critério.',
          top: true,
          left: false,
          timeout: 3000,
          color: 'error',
        });
      }
    } else {
      AlertService.show({
        name: 'basic-alert',
        text: 'Favor preencher os campos solicitados!',
        top: true,
        left: false,
        timeout: 3000,
        color: 'error',
      });
    }
  }

  public getDadosCard({ component }: IEventParam<IterableComponentRender>) {
    const parentIterable = Metadata.getInstance<IterableComponentRender>('iterable_parent');
    const id = component.name.split('_').pop();
    console.log("id", id);

    const parentIndex = parentIterable.datasource.data.findIndex((row: IDictionary) => row.DSTAGSTAREFA === id);
    console.log("iterable", parentIndex);
    const destructuring = parentIterable.datasource.data[parentIndex].list;
    console.log("n sei o que e", destructuring);
    component.datasource.data = destructuring;
  }

  public getDadosCriterios({ component }: IEventParam<IterableComponentRender>) {
    const parentIterable = Metadata.getInstance<IterableComponentRender>('iterable_parent');
    const nome = component.name.split('_');
    const idFilho = nome.pop();
    const idPai = nome.pop();
    const parentIndex = parentIterable.datasource.data.findIndex((row: IDictionary) => row.DSTAGSTAREFA === idPai);
    const filhoIndex = parentIterable.datasource.data[parentIndex].list.findIndex((row: IDictionary) => row.NRATIVIDADE === Number(idFilho));
    const dadoscard = parentIterable.datasource.data[parentIndex].list[filhoIndex].list;
    component.datasource.data = dadoscard;
  }
}
